%% Provided by Seyed Hamid Rezatofighi (The Australian National University)
% based on the following paper
% S. H. Rezatofighi, W. T. E. Pitkeathly, S. Gould, R. Hartley, K. Mele,
% W. E. Hughes, and James G. Burchfield, A Framework for Generating Realistic
% Synthetic Sequences of Total Internal Reflection Fluorescence Microscopy
% Images, ISBI 2013.

% Simulations in this framework consist of
% Five targets' categories: 1- Static targets (Donor organelle) 2- Docking targets
% 3- Existing targets with switching dynamics 4- Newborn targets with switching dynamics
% 5- Spawned targets with switching dynamics
% The targets in the first type (Donor organelle) are able to spawn new
% targets. Moreover, moving targets may merge to these targets.
% In addition, the simulation includes:
% Temporary disappearance, new born and dead targets, real background and
% spatio-temporally varying SNR.

close all
clear all
clc

%% ******************** Main Input Parameters *****************************
No=200; % Total number of the existing targets (excluding newborn and spawned targets).
ReC=[0.2 0.2]; % The proportion of total number of targets for each type ...
% [first type, second type] and third type=1-first type+second type;
Birth_Ratio=0.5;% Mean number of birth rate per frame based on Poisson distribution
if ReC(1)~=0 % For spawn phenomena, we need to have at least one Donor organelle
    Spawn_Ratio=0.5;% Mean number of spawn rate per frame based on Poisson distribution
end


Sp_res=158; %nm/pixel (The spatial resolution)
Depth_res=4; %nm/value (The spatial resolution for depth)
Temp_res=10; %fps (The temporal resolution)

Frame=60; % Number of frames
spot_size_Type(1,:)=[300 400];% [Minimum Size (nm),Range (nm)] for the static targets (Donor organelle)
spot_size_Type(2,:)=[250 200];% [Minimum Size (nm),Range (nm)] for the dynamic targets
swithch_treshold=[790 4740]; %[Minimum velocity (nm/S), Maximum velocity (nm/S)] that a targets
% with constant velocity dynamics can have. This range is used for switching...
% from constant velocity dynamics to Random walk and vice versa.

gamma=8690;% (nm/S) The coefficient for deforming shape of the targets based on their velocity
zeta=100; % (nm) decaying factor for relation between depth and intensity
Merging='Yes'; % The merging event may include the simulation ny answering 'Yes' or 'No'
bound_stop='Yes'; % The moving target cannot cross the boundary of cell (Mask) using this option
p_bern=0.5; %The probability that a target in depth die and cannot come back to surface


SNR=[3 10]; % Minimum (A) and Maximum (B) range of SNR [A B](Spatial range for SNR).
% SNR_changing_rate=repmat([1 1],Frame,1);% Rate of change of SNR per frame (for temporally constant SNR)
SNR_changing_rate=[exp(-(0:Frame-1)'/(Frame*8)) exp(-(0:Frame-1)'/(Frame*4))];% Rate of change of SNR per frame (for temporally varying SNR)
SNR_Frame=repmat(SNR,Frame,1).*SNR_changing_rate; % Minimum and maximum range of SNR for each frame.
% Remark: In this current implementation, the generated sequences have an
% SNR range close (not exactly equal) to the specified range.


%% ********************** Advanced Input Parameters ***********************
% ************* Number of targets & their starting frames *****************
Net=round(No*ReC); Net(end+1)=No-sum(Net);%Numbers of existing targets (excluding newborn and spawned targets)
SFET=ones(1,No);% Starting Frame for existing targets (These targets exist from the first frame)

% The number of newborn targets in each frame using pre-specified Birth_Ratio
Number_Birth=poissrnd(Birth_Ratio,1,Frame);
Frame_Birth=1:Frame;
Frame_Birth=Frame_Birth(Number_Birth>0);% Starting frame for newborn targets
Number_Birth=Number_Birth(Number_Birth>0);
NB=sum(Number_Birth);
SFB=[];
for i=1:length(Frame_Birth)
    SFB=[SFB repmat(Frame_Birth(i),1,Number_Birth(i))];
end

% The number of spawned targets in each frame using pre-specified Spawn_Ratio
if Net(1)~=0 % For spawn phenomena, we need to have at least one Donor organelle
    Number_Spawn=poissrnd(Spawn_Ratio,1,Frame);
    if Number_Spawn(1)~=0 % Spawning is avoided in the first frame
        Number_Spawn(2)=Number_Spawn(2)+Number_Spawn(1);
        Number_Spawn(1)=0;
    end
    Frame_Spawn=1:Frame;
    Frame_Spawn=Frame_Spawn(Number_Spawn>0); % Starting frame for spawned targets
    Number_Spawn=Number_Spawn(Number_Spawn>0);
    NS=sum(Number_Spawn);
    SFS=[];%Starting frame of Spawn
    SpI=[];%Spawn index from donor organelle
    for i=1:length(Frame_Spawn)
        SFS=[SFS repmat(Frame_Spawn(i),1,Number_Spawn(i))];
        SpI=[SpI randi(Net(1),1,Number_Spawn(i))]; % This determines that which donor organelle
        % spawns which spawned target
    end
else
    SFS=[];
    SpI=[];
end

TNot=[Net NB NS];% Total number of targets including existing, spawn and birth
SpawnIn=[zeros(1,sum(TNot)-NS) SpI]; % This determines that which target is spawned from
% which donor organelle. The zero elements mean that these targets are not spawned.
Start_Frame=[SFET SFB SFS];% Starting frame of all targets

% ************************ Dynamics models ********************************
T=1; % Sampling Interval
% Jittering dynamic model for donor organelle (static targets)
F1=[1 0;0 0];
F{1}=blkdiag(F1,F1,F1);% Random walk
q1=0.0001; % The process noise
Q{1}=blkdiag(q1*T^2*eye(2,2),q1*T^2*eye(2,2),q1*T^2*eye(2,2));% Random walk noise covariance

% Random walk dynamic model for docking targets and for the targets with switching
% dynamics such as existing, newborn and spawned targets
F{2}=blkdiag(F1,F1,F1);% Random walk
q2=0.7;% The process noise
Q{2}=blkdiag(q2*T^2*eye(2,2),q2*T^2*eye(2,2),q2*T^2*eye(2,2));% Random walk noise covariance

% Constant velocity dynamic model for the targets with switching
% dynamics such as existing, newborn and spawned targets.
F2=[1 T;0 1];
F{3}=blkdiag(F2,F2,F2);% Constant velocity
q0=0.0001;q3=0.7;% The process noise
Q{3}=blkdiag(q3*[T^3/3 T^2/2;T^2/2 T],q3*[T^3/3 T^2/2;T^2/2 T],q0*[T^3/3 T^2/2;T^2/2 T]);% Constant velocity noise covariance

Dyn_mod=[1 2 2 2 2;0 0 3 3 3];% Dynamics model (each column) for each target's type.

% Dynamic models for each target
Fot=zeros(size(Dyn_mod,1),sum(TNot));
TNot2=[0 TNot];
for i=1:length(TNot)
    for j=1:size(Dyn_mod,1)
        Fot(j,sum(TNot2(1:i))+1:sum(TNot2(1:i))+TNot2(i+1))=Dyn_mod(j,i);
    end
end

% ************************ Measurement models ****************************

% Shape parameters
Zigma(1,:)=[spot_size_Type(1,1)+spot_size_Type(1,2)*rand(1,TNot(1)) ...
    spot_size_Type(2,1)+spot_size_Type(2,2)*rand(1,sum(TNot)-TNot(1))];
%Standard deviation in X direction for some big static structures and some small dynamic structures
Zigma(2,:)=Zigma(1,:)-0.5*Sp_res+Sp_res*rand(1,sum(TNot));
%Standard deviation in y direction


% Real Backgrounds extracted using MPHD method.
TIRF=double(imread('Est_Back2_DS.tif'));
[u,v]=size(TIRF);
Background=uint8(zeros(u,v,Frame));
for f=1:Frame % Background
    Background(:,:,f)=imread('Est_Back2_DS.tif',f);
end
Backg=double(Background)/double(max(max(max(Background))));

% Loading the mask which shows the body of cell
BMask=imread('Mask2.tif');

% The parameters and conditions for SNR adjustment

Bmin=min(min(min(double(repmat((~BMask>0),[1 1 Frame]))+...
    Backg.*double(repmat((BMask>0),[1 1 Frame])))));...
    Bmax=max(max(max(Backg.*double(repmat((BMask>0),[1 1 Frame])))));
B_range=[Bmin Bmax]; % The range for the background's intensity (used for SNR adjustment)


Imin=exp(-round(0.6*zeta)/zeta); % The minimum SNR is for the targets located
% in the depth>round(0.6*zeta). The targets located below this depth
% have lower SNR because they are supposed to be disappeared.
Imax=1;
I_Range=[Imin Imax]; % The range for the targets' intensity (used for SNR adjustment)

% This condition is required to have a feasible SNR range using the
% current parameters and the background
if any(SNR_Frame(:,2)./SNR_Frame(:,1)<=sqrt(Imax/Imin))||any(SNR_Frame(:,2)./SNR_Frame(:,1)>=Imax/Imin*sqrt(Bmax/Bmin))
    error([num2str(sqrt(Imax/Imin)),'<SNR_Max/SNR_min<',num2str(Imax/Imin*sqrt(Bmax/Bmin)), ' for all frames']);
end

%% ********************** Track initialization ****************************
% This function generates initial state (position (x,y,z) and velocity (\dot(x),\dot(y),\dot(z))
% for each target and can be substituted by your own Initializer. We tried to generate
% realistic initial states.

werd=25; % The distance from the boundary of the cell for initialization of the positions
Int_Depth=[0 0 0 round(zeta) 0;round(0.05*zeta) round(0.2*zeta)...
    round(0.6*zeta) round(1.2*zeta) 0]; % Initial Depth for each category is chosen randomly from a position in
% [min_depth;max_depth]
Int_Depth_Velocity=[0 0 0 -8*round(0.04*zeta) 0]; % Initial velocity of each type in depth \dot(z)
Int_Spatial_Velocity=[0 -swithch_treshold(1) -swithch_treshold(2) -swithch_treshold(2) -swithch_treshold(1);...
    0.001 swithch_treshold(1) swithch_treshold(2) swithch_treshold(2) swithch_treshold(2)]; % Initial spatial velocity
% of each type (\dot(x),\dot(y)) is chosen randomly from a velocity range [min_Spatial_Velocity;max_Spatial_Velocity]
Int_Spawn_displacement=[2*Sp_res;4*Sp_res]; % Initial location for spawned targets
% is equal to the location of its spawner(x,y,z) + (d_x,d_y,0). d_x and d_y
% are randomly selected from the specified range
% Int_Spawn_displacement=[2*Sp_res;4*Sp_res];

X0=Initializer(TNot,Sp_res,Depth_res,Temp_res,BMask,werd,Int_Spatial_Velocity,...
    Int_Depth,Int_Depth_Velocity,Int_Spawn_displacement);
%% *************************** Dynamic Model ******************************
% This function simulates the targets' trajectories using the pre-specified
% linear dynamic models (random walk and constant velocity). To be more
% realistic, the dynamic model in depth (z) is not completely a linear
% model and in this dimension, the targets show some non-linear behavior
% close to the cell surface z<<zeta and in very deep depth (z>1.2*zeta).

% you can use your own Trajectory_Simulator if it is more realistic than
% the current framework.

% Output for this function:
% Xdy: The simulated state for each target for each frame
% Frdy: The starting and last frame for each target
% MergeIn: It includes the index representing which target merged to which target
% Fdy: The transition matrix F for each target in each frame
% Qdy: The covariance matrix Q for each target in each frame
% Dyn: Dynamic index for each target in each frame

[Xdy,Frdy,MergeIn,Fdy,Qdy,Dyn]=Trajectory_Simulator...
    (F,Q,Fot,TNot,X0,Frame,Start_Frame,SpawnIn,swithch_treshold,Merging,BMask,...
    Sp_res,Temp_res,Depth_res,zeta,p_bern,werd,bound_stop);

Mask=(BMask>0);
Bound = cell2mat(bwboundaries(Mask,8,'noholes'));
imshow(Mask)
hold on
plot(Bound(:,2),Bound(:,1),'.r')
title('The dynamics of each target')
colorord0=.25+.75*rand(sum(TNot),3);
for uu=1:Frame
    for ji=1:length(Xdy)
        if uu>=Frdy{1,ji}(1)&&uu<=Frdy{1,ji}(2)
            hold on
            plot(Xdy{1,ji}(3,1:uu-Frdy{1,ji}(1)+1),Xdy{1,ji}(1,1:uu-Frdy{1,ji}(1)+1),'.','Color',colorord0(ji,:))
        end
    end
    pause(0.1)
end
figure,
xlabel('Frame')
ylabel('Depth (nm)')
title('The dynamics of each target in depth')
for ji=1:length(Xdy)
    hold on
    plot(Frdy{1,ji}(1):Frdy{1,ji}(2),Xdy{1,ji}(5,1:end)*Depth_res,'-','Color',colorord0(ji,:))
end
%% ************************ Measurement Model *****************************
% This is the main function and the outputs are
% I: Generated sequences including spots without background and noise
% In: Generated sequences including spots, background and noise. Because
% the intensity of these sequences are not normalized between 0 and 255 for
% uint8 or 0 and 1 for double, the SNR range is close the pre-specified value
% In_norm: Generated sequences including spots, background and noise. The intensity
% of these sequences are normalized between 0 and 1 for being saved as a uint8 sequences.
% Therefore, the calculated SNR for these sequences should be multiplied by Norm_SNR
% CI and CB: They are the coefficients () for each frame for combining the background
% and the generated spots to have a SNR equal to the pre-specified range.
% SNR_Targets: The calculated SNR for each target and for each frame in the
% "In" sequences
% SNR_Tt: it is exactly same as SNR_Targets. Just converted from cell to the
% vector. You can compute the SNR mean and variance using it.
% Norm_SNR: if you want to use In_norm as your generated sequences, you
% must multiply SNR_Tt or SNR_Targets by this value to have the SNR for In_norm

[I,In,In_norm,CI,CB,SNR_Targets,SNR_Tt,Norm_SNR]=Sequence_Generator...
    (Zigma,gamma,I_Range,B_range,zeta,Sp_res,Depth_res,Temp_res,SNR_Frame,Backg,Frame,Xdy,Frdy);
figure,
for u=1:100
    for uu=1:Frame
        imshow(In_norm(:,:,uu),'InitialMagnification','fit')
        pause(1/Temp_res)
    end
end


